#include "booklistitemdelegate.h"
#include "defines.h"
#include "BookListItem.h"
#include "bookmoocherdata.h"

#include <QPainter>
#include <QSize>
#include <QStyleOption>
#include <QStyleOptionViewItemV2>
#include <QLocale>

BookListItemDelegate::BookListItemDelegate(QObject *parent) :
    QStyledItemDelegate(parent)
{
}

void BookListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const {

	if (!index.isValid()) {
		QStyledItemDelegate::paint(painter, option, index);
		return;
	}

	const QStyleOptionViewItemV2 *option2 = qstyleoption_cast<const QStyleOptionViewItemV2 *>(&option);

	painter->save();

	painter->setClipRect(option.rect);

	if (option.state & QStyle::State_Selected) {
		painter->fillRect(option.rect, option.palette.highlight());
		painter->setBrush(option.palette.highlight());
		painter->setPen(option.palette.highlightedText().color());
	} else {
		if (option2->features & QStyleOptionViewItemV2::Alternate) {
			painter->fillRect(option.rect, option.palette.alternateBase() );
			painter->setBrush(option.palette.alternateBase());
			painter->setPen(option.palette.text().color());
		} else {
			painter->fillRect(option.rect, option.palette.base() );
			painter->setBrush(option.palette.base());
			painter->setPen(option.palette.text().color());
		}
	}


	BookListItem feed = index.data().value<BookListItem>();

	// Testata
	QFont f = option.font;
	f.setPointSize( f.pointSize() + 2 );
	if(feed.is_new) f.setBold(true);
	painter->setFont(f);

	QFontMetrics fm_header(f);

	QString titolo = fm_header.elidedText( QString::number(index.row() + 1) + " - " + feed.title, Qt::ElideRight, option.rect.width() );

	int y = option.rect.y() + fm_header.height();

	painter->drawText(option.rect, titolo);

	painter->setFont(option.font);
	// Autore
	painter->drawText(option.rect.left(), y, option.rect.width(), option.fontMetrics.height(),
					  Qt::TextSingleLine, option.fontMetrics.elidedText( tr("by ") + feed.author + tr(" - Shared by: ") + feed.username, Qt::ElideRight, option.rect.width() ));

	y += option.fontMetrics.height();

	// Altri dati
	painter->drawText(option.rect.left(), y, option.rect.width(), option.fontMetrics.height(),
					  Qt::TextSingleLine, option.fontMetrics.elidedText( getDateAndCountryText(index), Qt::ElideRight, option.rect.width() ));


	painter->restore();
}

QSize BookListItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const {

	if (!index.isValid()) return QStyledItemDelegate::sizeHint(option,index);

	int w = 100, h = 0;

	// Testata: bold solo se il libro è nuovo
	QFont f = option.font;
	f.setPointSize( f.pointSize() + 2 );
	if (index.data().value<BookListItem>().is_new) f.setBold(true);
	QFontMetrics fm(f);

	h = fm.height();

	// altre due righe, font di sistema
	h += option.fontMetrics.height() * 2;

	return QSize(w, h + 8);
}

QString BookListItemDelegate::getDateAndCountryText(const QModelIndex &index) const {
	BookListItem feed = index.data().value<BookListItem>();
	return tr("On ") + QLocale::system().toString(feed.date, QLocale::ShortFormat) + tr(" in ") + BookMoocherData::getCountries().value(feed.country);
}

QString BookListItemDelegate::getGenresText(const QModelIndex &index) const {
	// Generi
	BookListItem feed = index.data().value<BookListItem>();
	QString genres = tr("Genres:");
	if (feed.genres.isEmpty()) genres += tr(" N/A");
	else {
		QStringListIterator gi(feed.genres);
		while (gi.hasNext()) {
			genres += BookMoocherData::getGenres().value( gi.next() );
			if (gi.hasNext()) genres +=  ", ";
		}
	}
	return genres;
}
